@@ -1,13 +1,13 @@
 from logging import getLogger
 
-from django.contrib.auth import authenticate as auth, login
+from django.contrib.auth import authenticate as auth, login, get_backends
 from django.http import Http404
 from django.shortcuts import redirect
 
-from django_social_user import registered_networks, options
-from django_social_user.exceptions import (
+from django_social_user import options
+from django_social_user.exceptions import (DoNotAuthenticate,
     SocialOauthDictFailed, MissingRequiredSetting)
-from django_social_user.signals import (
+from django_social_user.signals import (django_social_user_pre_callback,
     django_social_user_pre_auth, django_social_user_post_callback)
 
 logger = getLogger('django_social_user:views')
@@ -18,14 +18,17 @@ SK_REQUEST_TOKEN = 'django_social_user:oauth_request_token'
 
 def get_network_backend_or_404(network):
     """
-    Fetch the network from the registered list or throw a 404 exception.
+    Fetch the network backend from django's list of backends or throw
+    a 404 exception.
     """
-    backend = registered_networks.get(network)
-    if not backend:
-        logger.exception(
-            'Attempting to authenticate unregistered backend %s' % network)
-        raise Http404
-    return backend
+    if network:
+        for backend in get_backends():
+            if network == getattr(backend, 'network', None):
+                return backend
+
+    logger.exception(
+        'Attempting to authenticate unregistered backend %s' % network)
+    raise Http404
 
 
 def authenticate(request, network):
@@ -74,22 +77,33 @@ def callback(request, network):
             options.REDIRECT_ON_ACCESS_TOKEN_FAILURE,
             options.KEY_ACCESS_TOKEN_FAILURE)
 
-    # if the user is already authenticated, pass into the backend,
-    #   so that the social identity can be associated to the user
-    user = request.user if request.user.is_authenticated() else None
+    responses = django_social_user_pre_callback.send_robust(
+                                    None,
+                                    request=request,
+                                    network=network,
+                                    access_token=access_token,
+                                    access_token_expires=access_token_expires)
 
-    user = auth(
-        network=network, access_token=access_token,
-        access_token_expires=access_token_expires, user=user)
+    # Check to see if the pre_callback signal raised an exception to skip
+    # the authentication backend.
+    if not any([isinstance(response, DoNotAuthenticate)
+                for receiver, response in responses]):
+        # If the user is already authenticated, pass into the backend
+        # so that the social identity can be associated to the user.
+        user = request.user if request.user.is_authenticated() else None
 
-    if user is not None:
-        if user.is_active:
-            login(request, user)
+        user = auth(
+            network=network, access_token=access_token,
+            access_token_expires=access_token_expires, user=user)
+
+        if user is not None:
+            if user.is_active:
+                login(request, user)
+            else:
+                # todo: handle this better
+                raise SocialOauthDictFailed('User is not active')
         else:
-            # todo: handle this better
-            raise SocialOauthDictFailed('User is not active')
-    else:
-        raise SocialOauthDictFailed('User not returned from authorization')
+            raise SocialOauthDictFailed('User not returned from authorization')
 
     django_social_user_post_callback.send(None, request=request)
 
